\name{A2_calcG0}
\Rdversion{1.2}
\encoding{UTF-8}
\alias{calcG0}

\title{Irradiation and irradiance on the horizontal plane.}

\description{
  This function obtains the global, diffuse and direct irradiation and
  irradiance on the horizontal plane from the values of \emph{daily} and
  \emph{intradaily} global irradiation on the horizontal plane.
  It makes use of the functions \code{\link{calcSol}},
  \code{\link{fCompD}}, \code{\link{fCompI}}, \code{\link{fBTd}}
  and \code{\link{readBDd}} (or equivalent).

  Besides, if information about maximum and minimum temperatures values are available it obtains a series of temperature values with \code{\link{fTemp}}.
}

\usage{

calcG0(lat, modeRad = 'prom', dataRad,
       sample = 'hour', keep.night = TRUE,
       sunGeometry = 'michalsky',
       corr, f, ...)
}

\arguments{
  \item{lat}{numeric, latitude (degrees) of the point of the Earth where calculations are needed. It is positive for locations above the Equator.}

  \item{modeRad}{A character string, describes the kind of source data of the
    global irradiation and ambient temperature.

    It can be \code{modeRad = 'prom'} for monthly mean calculations. With
    this option, a set of 12 values inside \code{dataRad} must be
    provided, as defined in \code{\link{readG0dm}}.

    \code{modeRad = 'aguiar'} uses a set of 12 monthly average values
    (provided with \code{dataRad}) and produces a synthetic daily
    irradiation time series following the procedure by Aguiar etal. (see
    reference below).

    If \code{modeRad = 'bd'} the information of \emph{daily} irradiation is read from a
    file, a \code{data.table} defined by \code{dataRad}, a \code{zoo} or a
    \code{Meteo} object. (See \code{\link{readBDd}}, \code{\link{dt2Meteo}}
    and \code{\link{zoo2Meteo}} for details).

    If \code{modeRad = 'bdI'} the information of \emph{intradaily} irradiation is read from a
    file, a \code{data.table} defined by \code{dataRad}, a \code{zoo} or a
    \code{Meteo} object. (See \code{\link{readBDi}},
    \code{\link{dt2Meteo}} and \code{\link{zoo2Meteo}} for details).
  }

  \item{dataRad}{\itemize{
      
      \item{If \code{modeRad = 'prom'} or \code{modeRad = 'aguiar'}, a
	numeric with 12 values or a named list whose components will be
	processed with \code{\link{readG0dm}}.}
      
      \item{If \code{modeRad = 'bd'} a character (name of the file to be
    read with \code{readBDd}), a \code{data.table} (to be processed with
    \code{dt2Meteo}), a \code{zoo} (to be processed with
    \code{zoo2Meteo}), a \code{Meteo} object, or a \code{list} as
    defined by \code{\link{readBDd}}, \code{\link{dt2Meteo}} or
    \code{\link{zoo2Meteo}}. The resulting object will include a column
    named \code{Ta}, with information about ambient temperature.}

        \item{If \code{modeRad = 'bdI'} a character (name of the file to
    be read with \code{readBDi}), a \code{data.table} (to be processed
    with \code{dt2Meteo}), a \code{zoo} (to be processed with
    \code{zoo2Meteo}), a \code{Meteo} object, or a \code{list} as
    defined by \code{\link{readBDi}}, \code{\link{dt2Meteo}} or
    \code{\link{zoo2Meteo}}. The resulting object will include a column
    named \code{Ta}, with information about ambient temperature.}
  }}

  \item{sample}{\code{character}, containing one of ‘"sec"’, ‘"min"’,
    ‘"hour"’.  This can optionally be preceded by a (positive or
    negative) integer and a space, or followed by ‘"s"’ (used by
    \code{\link{seq.POSIXt}}). It is not used when \code{modeRad = "bdI"}.}

  \item{keep.night}{\code{logical}. When it is TRUE (default) the
    time series includes the night.}

  \item{sunGeometry}{\code{character}, method for the sun geometry
    calculations. See \code{\link{calcSol}}, \code{\link{fSolD}} and \code{\link{fSolI}}.}

  \item{corr}{A character, the correlation between the fraction of diffuse irradiation and the
    clearness index to be used.

    With this version several options are available, as described in
    \code{\link{corrFdKt}}. For example, the \code{\link{FdKtPage}} is
    selected with \code{corr = 'Page'} while the \code{\link{FdKtCPR}} with
    \code{corr = 'CPR'}.

    If \code{corr = 'user'} the use of a correlation
    defined by a function \code{f} is possible.

    If \code{corr = 'none'} the object defined by \code{dataRad} should include
    information about global, diffuse and direct daily irradiation with columns named 
    \code{G0d}, \code{D0d} and \code{B0d}, respectively
    (or \code{G0}, \code{D0} and \code{B0} if \code{modeRad = 'bdI'}).
    If \code{corr} is missing, then it is internally set to \code{CPR}
    when \code{modeRad = 'bd'}, to \code{Page}
    when \code{modeRad = 'prom'} and to \code{BRL} when \code{modeRad = 'bdI'}.}

  \item{f}{A function defininig a correlation between the fraction of
    diffuse irradiation and the clearness index. It is only neccessary
    when \code{corr = 'user'}}

  \item{...}{Additional arguments for \code{\link{fCompD}} or \code{\link{fCompI}}}
}

\value{A \code{G0} object.}

\references{\itemize{
    \item Perpiñán, O, Energía Solar Fotovoltaica, 2015.
    (\url{https://oscarperpinan.github.io/esf/})

    \item Perpiñán, O. (2012), "solaR: Solar Radiation and Photovoltaic
    Systems with R", Journal of Statistical Software, 50(9), 1-32,
    \doi{10.18637/jss.v050.i09}

    \item Aguiar, Collares-Pereira and Conde, "Simple procedure for
  generating sequences of daily radiation values using a library of
  Markov transition matrices", Solar Energy, Volume 40, Issue 3, 1988, Pages 269–279}
  }

  \author{Oscar Perpiñán Lamigueiro,
    Francisco Delgado López.}

\seealso{
  \code{\link{calcSol}},
  \code{\link{fCompD}},
  \code{\link{fCompI}},
  \code{\link{readG0dm}},
  \code{\link{readBDd}},
  \code{\link{readBDi}},
  \code{\link{dt2Meteo}},
  \code{\link{corrFdKt}}.
}
\examples{
library("data.table")
\dontshow{setDTthreads(2)}

G0dm <- c(2.766, 3.491, 4.494, 5.912, 6.989, 7.742, 7.919, 7.027, 5.369, 3.562, 2.814, 2.179)*1000;
Ta <- c(10, 14.1, 15.6, 17.2, 19.3, 21.2, 28.4, 29.9, 24.3, 18.2, 17.2, 
  15.2)

g0 <- calcG0(lat = 37.2, modeRad = 'prom', dataRad = list(G0dm = G0dm, Ta = Ta))
print(g0)
xyplot(g0)

## Aguiar et al.

g0 <- calcG0(lat = 37.2, modeRad = 'aguiar', dataRad = G0dm)
print(g0)
xyplot(g0)

##Now the G0I component of g0 is used as
##the bdI argument to calcG0 in order to
##test the intradaily correlations of fd-kt

BDi = as.data.tableI(g0)
BDi$Ta = 25 ##Information about temperature must be contained in BDi

g02 <- calcG0(lat = 37.2, 
            modeRad = 'bdI', 
            dataRad = list(lat = 37.2, file = BDi), 
            corr = 'none')

print(g02)

g03 <- calcG0(lat = 37.2, 
            modeRad = 'bdI', 
            dataRad = list(lat = 37.2, file = BDi), 
            corr = 'BRL')
print(g03)

xyplot(Fd ~ Kt, data = g03, pch = 19, alpha = 0.3)
}

\keyword{constructors}

